@@ -9,6 +9,7 @@ import json
 import copy
 import logging
 import re
+from uuid import UUID
 from datetime import datetime
 from pathlib import Path
 from typing import Union
@@ -19,6 +20,7 @@ import pandas as pd
 from brainbox.core import Bunch
 from brainbox.io import parquet
 from ibllib.io import jsonable
+from ibllib.exceptions import ALFObjectNotFound
 from . import files
 
 _logger = logging.getLogger('ibllib')
@@ -179,7 +181,7 @@ def load_file_content(fil):
     if fil.suffix == '.jsonable':
         return jsonable.read(fil)
     if fil.suffix == '.npy':
-        return np.load(file=fil)
+        return np.load(file=fil, allow_pickle=True)
     if fil.suffix == '.pqt':
         return parquet.load(fil)
     if fil.suffix == '.ssv':
@@ -197,7 +199,9 @@ def _ls(alfpath, object=None, **kwargs):
     :return: lists of pathlib.Path for each file and list of corresponding attributes
     """
     alfpath = Path(alfpath)
-    if alfpath.is_dir():
+    if not alfpath.exists():
+        files_alf = None
+    elif alfpath.is_dir():
         if object is None:
             # List all ALF files
             files_alf, attributes = files.filter_by(alfpath)
@@ -211,7 +215,7 @@ def _ls(alfpath, object=None, **kwargs):
     # raise error if no files found
     if not files_alf:
         err_str = 'object "%s" ' % object if object else 'ALF files'
-        raise FileNotFoundError('No {} found in {}'.format(err_str, str(alfpath)))
+        raise ALFObjectNotFound(f'No {err_str} found in {alfpath}')
 
     return [alfpath.joinpath(f) for f in files_alf], attributes
 
@@ -228,7 +232,7 @@ def exists(alfpath, object, attributes=None, **kwargs):
     # if the object is not found, return False
     try:
         _, attributes_found = _ls(alfpath, object, **kwargs)
-    except FileNotFoundError:
+    except (FileNotFoundError, ALFObjectNotFound):
         return False
 
     # if object found and no attribute provided, True
@@ -380,7 +384,9 @@ def add_uuid_string(file_path, uuid):
     if isinstance(uuid, str) and not is_uuid_string(uuid):
         raise ValueError('Should provide a valid UUID v4')
     uuid = str(uuid)
-    file_path = Path(file_path)
+    # NB: Only instantiate as Path if not already a Path, otherwise we risk changing the class
+    if isinstance(file_path, str):
+        file_path = Path(file_path)
     name_parts = file_path.stem.split('.')
     if uuid == name_parts[-1]:
         _logger.warning(f'UUID already found in file name: {file_path.name}: IGNORE')
@@ -390,7 +396,8 @@ def add_uuid_string(file_path, uuid):
 
 def is_uuid_string(string: str) -> bool:
     """
-    Bool test for uuid version 4
+    Bool test for randomly generated hexadecimal uuid validity
+    NB: uuid must be hyphen separated
     """
     if string is None:
         return False
@@ -403,6 +410,20 @@ def is_uuid_string(string: str) -> bool:
         return False
 
 
+def is_uuid(uuid: Union[str, int, bytes, UUID]) -> bool:
+    """Bool test for randomly generated hexadecimal uuid validity
+    Unlike `is_uuid_string`, this function accepts UUID objects
+    """
+    if not isinstance(uuid, (UUID, str, bytes, int)):
+        return False
+    elif not isinstance(uuid, UUID):
+        try:
+            uuid = UUID(uuid) if isinstance(uuid, str) else UUID(**{type(uuid).__name__: uuid})
+        except ValueError:
+            return False
+    return isinstance(uuid, UUID) and uuid.version == 4
+
+
 def _isdatetime(s: str) -> bool:
     try:
         datetime.strptime(s, '%Y-%m-%d')
