@@ -88,7 +88,8 @@ def get_sessions(subj, date=None, one=None):
 
     if date is None:
         # compute from yesterday
-        latest_sess = (datetime.date.today() - datetime.timedelta(days=1)).strftime("%Y-%m-%d")
+        specified_date = (datetime.date.today() - datetime.timedelta(days=1))
+        latest_sess = specified_date.strftime("%Y-%m-%d")
         latest_minus_week = (datetime.date.today() -
                              datetime.timedelta(days=8)).strftime("%Y-%m-%d")
     else:
@@ -120,10 +121,6 @@ def get_sessions(subj, date=None, one=None):
             trials_ = one.load_object(sessions[n]['url'].split('/')[-1], 'trials')
 
             if trials_:
-                trials_.probabilityLeft, = one.load(sessions[n]['url'].split('/')[-1],
-                                                    'trials.probabilityLeft')
-                trials_.response_times, = one.load(sessions[n]['url'].split('/')[-1],
-                                                   'trials.response_times')
                 task_protocol.append(re.search('tasks_(.*)Choice',
                                      sessions[n]['task_protocol']).group(1))
                 sess_dates.append(sessions[n]['start_time'][:10])
@@ -135,10 +132,6 @@ def get_sessions(subj, date=None, one=None):
             trials_ = one.load_object(sessions[n]['url'].split('/')[-1], 'trials')
 
             if trials_:
-                trials_.probabilityLeft, = one.load(sessions[n]['url'].split('/')[-1],
-                                                    'trials.probabilityLeft')
-                trials_.response_times, = one.load(sessions[n]['url'].split('/')[-1],
-                                                   'trials.response_times')
                 task_protocol.append(re.search('tasks_(.*)Choice',
                                      sessions[n]['task_protocol']).group(1))
                 sess_dates.append(sessions[n]['start_time'][:10])
@@ -385,7 +378,7 @@ def get_signed_contrast(trials):
     returns: array of signed contrasts in percent, where -ve values are on the left
     """
     # Replace NaNs with zeros, stack and take the difference
-    contrast = np.nan_to_num(np.c_[trials.contrastLeft, trials.contrastRight])
+    contrast = np.nan_to_num(np.c_[trials['contrastLeft'], trials['contrastRight']])
     return np.diff(contrast).flatten() * 100
 
 
@@ -400,7 +393,7 @@ def compute_performance_easy(trials):
     """
     signed_contrast = get_signed_contrast(trials)
     easy_trials = np.where(np.abs(signed_contrast) >= 50)[0]
-    return np.sum(trials.feedbackType[easy_trials] == 1) / easy_trials.shape[0]
+    return np.sum(trials['feedbackType'][easy_trials] == 1) / easy_trials.shape[0]
 
 
 def compute_n_trials(trials):
@@ -411,7 +404,7 @@ def compute_n_trials(trials):
     :type trials: dict
     returns: int containing number of trials in session
     """
-    return trials.choice.shape[0]
+    return trials['choice'].shape[0]
 
 
 def compute_psychometric(trials, signed_contrast=None, block=None):
@@ -434,6 +427,9 @@ def compute_psychometric(trials, signed_contrast=None, block=None):
     else:
         block_idx = trials.probabilityLeft == block
 
+    if not np.any(block_idx):
+        return np.nan * np.zeros(4)
+
     contrasts, n_contrasts = np.unique(signed_contrast[block_idx], return_counts=True)
     rightward = trials.choice == -1
     # Calculate the proportion rightward for each contrast type
