@@ -11,7 +11,7 @@ from scipy import signal
 
 import alf.io
 from brainbox.core import Bunch
-from brainbox.metrics import quick_unit_metrics
+from brainbox.metrics.single_units import spike_sorting_metrics
 from ibllib.ephys import sync_probes
 from ibllib.io import spikeglx, raw_data_loaders
 import ibllib.dsp as dsp
@@ -155,13 +155,13 @@ def validate_ttl_test(ses_path, display=False):
 
     # get the synchronization fronts (from the raw binary if necessary)
     ephys_fpga.extract_sync(session_path=ses_path, overwrite=False)
-    rawsync, sync_map = ephys_fpga._get_main_probe_sync(ses_path)
+    rawsync, sync_map = ephys_fpga.get_main_probe_sync(ses_path)
     last_time = rawsync['times'][-1]
 
     # get upgoing fronts for each
     sync = Bunch({})
     for k in sync_map:
-        fronts = ephys_fpga._get_sync_fronts(rawsync, sync_map[k])
+        fronts = ephys_fpga.get_sync_fronts(rawsync, sync_map[k])
         sync[k] = fronts['times'][fronts['polarities'] == 1]
     wheel = ephys_fpga.extract_wheel_sync(rawsync, chmap=sync_map)
 
@@ -220,7 +220,7 @@ def validate_ttl_test(ses_path, display=False):
     return ok
 
 
-def unit_metrics_ks2(ks2_path=None, m=None, save=True):
+def spike_sorting_metrics_ks2(ks2_path=None, m=None, save=True):
     """
     Given a path containing kilosort 2 output, compute quality metrics and optionally save them
     to a clusters_metric.csv file
@@ -234,28 +234,26 @@ def unit_metrics_ks2(ks2_path=None, m=None, save=True):
                                                 'directory, or a phylib `TemplateModel` object'
     # create phylib `TemplateModel` if not given
     m = phy_model_from_ks2_path(ks2_path) if None else m
-    # compute metrics and convert to `DataFrame`
-    r = pd.DataFrame(quick_unit_metrics(m.spike_clusters, m.spike_times, m.amplitudes, m.depths))
-
+    c, drift = spike_sorting_metrics(m.spike_times, m.spike_clusters, m.amplitudes, m.depths)
     #  include the ks2 cluster contamination if `cluster_ContamPct` file exists
     file_contamination = ks2_path.joinpath('cluster_ContamPct.tsv')
     if file_contamination.exists():
         contam = pd.read_csv(file_contamination, sep='\t')
         contam.rename(columns={'ContamPct': 'ks2_contamination_pct'}, inplace=True)
-        r = r.set_index('cluster_id', drop=False).join(contam.set_index('cluster_id'))
+        c = c.set_index('cluster_id', drop=False).join(contam.set_index('cluster_id'))
 
     #  include the ks2 cluster labels if `cluster_KSLabel` file exists
     file_labels = ks2_path.joinpath('cluster_KSLabel.tsv')
     if file_labels.exists():
         ks2_labels = pd.read_csv(file_labels, sep='\t')
         ks2_labels.rename(columns={'KSLabel': 'ks2_label'}, inplace=True)
-        r = r.set_index('cluster_id', drop=False).join(ks2_labels.set_index('cluster_id'))
+        c = c.set_index('cluster_id', drop=False).join(ks2_labels.set_index('cluster_id'))
 
     if save:
         #  the file name contains the label of the probe (directory name in this case)
-        r.to_csv(ks2_path.joinpath('cluster_metrics.csv'))
+        c.to_csv(ks2_path.joinpath('cluster_metrics.csv'))
 
-    return r
+    return c
 
 
 def phy_model_from_ks2_path(ks2_path, bin_path, bin_file=None):
@@ -346,7 +344,7 @@ def qc_fpga_task(fpga_trials, alf_trials):
         np.abs(
             fpga_trials['stimOff_times'] - fpga_trials['valveOpen_times'] - VALVE_STIM_OFF_DELAY
         ),
-        VALVE_STIM_OFF_JITTER, out=np.ones(ntrials, dtype=np.bool),
+        VALVE_STIM_OFF_JITTER, out=np.ones(ntrials, dtype=bool),
         where=~np.isnan(fpga_trials['valveOpen_times']))
 
     # iti_in whithin 0.01 sec of stimOff
@@ -359,7 +357,7 @@ def qc_fpga_task(fpga_trials, alf_trials):
         np.abs(
             fpga_trials['stimOff_times'] - fpga_trials['errorCue_times'] - ERROR_STIM_OFF_DELAY
         ),
-        ERROR_STIM_OFF_JITTER, out=np.ones(ntrials, dtype=np.bool),
+        ERROR_STIM_OFF_JITTER, out=np.ones(ntrials, dtype=bool),
         where=~np.isnan(fpga_trials['errorCue_times']))
 
     """
@@ -398,7 +396,7 @@ def _qc_from_path(sess_path, display=True):
     raw_trials = raw_data_loaders.load_data(sess_path)
     tmax = raw_trials[-1]['behavior_data']['States timestamps']['exit_state'][0][-1] + 60
 
-    sync, chmap = ephys_fpga._get_main_probe_sync(sess_path, bin_exists=False)
+    sync, chmap = ephys_fpga.get_main_probe_sync(sess_path, bin_exists=False)
     _ = ephys_fpga.extract_all(sess_path, output_path=temp_alf_folder, save=True)
     # check that the output is complete
     fpga_trials = ephys_fpga.extract_behaviour_sync(sync, output_path=temp_alf_folder, tmax=tmax,
