@@ -1,5 +1,4 @@
 import logging
-import cv2
 import numpy as np
 from pkg_resources import parse_version
 
@@ -70,7 +69,7 @@ class ProbabilityLeft(BaseBpodTrialsExtractor):
     save_names = '_ibl_trials.probabilityLeft.npy'
     var_names = 'probabilityLeft'
 
-    def _extract(self):
+    def _extract(self, **kwargs):
         return np.array([t['stim_probability_left'] for t in self.bpod_trials])
 
 
@@ -314,7 +313,7 @@ class GoCueTimes(BaseBpodTrialsExtractor):
     Get trigger times of goCue from state machine.
 
     Current software solution for triggering sounds uses PyBpod soft codes.
-    Delays can be in the order of 10's of ms. This is the time when the command
+    Delays can be in the order of 10-100s of ms. This is the time when the command
     to play the sound was executed. To measure accurate time, either getting the
     sound onset from the future microphone OR the new xonar soundcard and
     setup developed by Sanworks guarantees a set latency (in testing).
@@ -378,7 +377,6 @@ class IncludedTrials(BaseBpodTrialsExtractor):
 
 
 class ItiInTimes(BaseBpodTrialsExtractor):
-    save_names = '_ibl_trials.itiIn_times.npy'
     var_names = 'itiIn_times'
 
     def _extract(self):
@@ -393,7 +391,6 @@ class ItiInTimes(BaseBpodTrialsExtractor):
 
 
 class ErrorCueTriggerTimes(BaseBpodTrialsExtractor):
-    save_names = '_ibl_trials.errorCueTrigger_times.npy'
     var_names = 'errorCueTrigger_times'
 
     def _extract(self):
@@ -409,7 +406,6 @@ class ErrorCueTriggerTimes(BaseBpodTrialsExtractor):
 
 
 class StimFreezeTriggerTimes(BaseBpodTrialsExtractor):
-    save_names = '_ibl_trials.stimFreeze_times.npy'
     var_names = 'stimFreezeTrigger_times'
 
     def _extract(self):
@@ -454,7 +450,6 @@ class StimFreezeTriggerTimes(BaseBpodTrialsExtractor):
 
 
 class StimOffTriggerTimes(BaseBpodTrialsExtractor):
-    save_names = '_ibl_trials.stimOffTrigger_times.npy'
     var_names = 'stimOffTrigger_times'
 
     def _extract(self):
@@ -507,11 +502,11 @@ class StimOnTimes(BaseBpodTrialsExtractor):
 
     def _extract(self):
         """
-        Find the time of the statemachine command to turn on hte stim
+        Find the time of the state machine command to turn on the stim
         (state stim_on start or rotary_encoder_event2)
-        Find the next frame change from the photodiodeafter that TS.
+        Find the next frame change from the photodiode after that TS.
         Screen is not displaying anything until then.
-        (Frame changes are in BNC1High and BNC1Low)
+        (Frame changes are in BNC1 High and BNC1 Low)
         """
         # Version check
         if version.ge(self.settings['IBLRIG_VERSION_TAG'], '5.0.0'):
@@ -589,8 +584,8 @@ class StimOnTimes(BaseBpodTrialsExtractor):
                 bnc_l.append(np.array([np.NINF]))
 
         stim_on = np.array(stim_on)
-        bnc_h = np.array(bnc_h)
-        bnc_l = np.array(bnc_l)
+        bnc_h = np.array(bnc_h, dtype=object)
+        bnc_l = np.array(bnc_l, dtype=object)
 
         count_missing = 0
         stimOn_times = np.zeros_like(stim_on)
@@ -615,9 +610,8 @@ class StimOnOffFreezeTimes(BaseBpodTrialsExtractor):
     """
     Extracts stim on / off and freeze times from Bpod BNC1 detected fronts
     """
-    save_names = ["_ibl_trials.stimOn_times.npy", "_ibl_trials.stimOff_times.npy",
-                  "_ibl_trials.stimFreeze_times.npy"]
-    var_names = ['stimOn_times', 'stimOff_times', 'stimFreeze_times']
+    save_names = ("_ibl_trials.stimOn_times.npy", None, None)
+    var_names = ('stimOn_times', 'stimOff_times', 'stimFreeze_times')
 
     def _extract(self):
         choice = Choice(self.session_path).extract(
@@ -653,96 +647,7 @@ class StimOnOffFreezeTimes(BaseBpodTrialsExtractor):
         return stimOn_times, stimOff_times, stimFreeze_times
 
 
-class CameraTimestamps(BaseBpodTrialsExtractor):
-    """
-    Get the camera timestamps from the Bpod
-
-    The camera events are logged only during the events not in between, so the times need
-    to be interpolated
-    """
-    save_names = '_ibl_leftCamera.times.npy'
-    var_names = 'camera_timestamps'
-
-    def _extract(self):
-        ntrials = len(self.bpod_trials)
-
-        cam_times = []
-        n_frames = 0
-        n_out_of_sync = 0
-        for ind in np.arange(ntrials):
-            # get upgoing and downgoing fronts
-            pin = np.array(self.bpod_trials[ind]['behavior_data']
-                           ['Events timestamps'].get('Port1In'))
-            pout = np.array(self.bpod_trials[ind]['behavior_data']
-                            ['Events timestamps'].get('Port1Out'))
-            # some trials at startup may not have the camera working, discard
-            if np.all(pin) is None:
-                continue
-            # if the trial starts in the middle of a square, discard the first downgoing front
-            if pout[0] < pin[0]:
-                pout = pout[1:]
-            # same if the last sample is during an upgoing front, always
-            # put size as it happens last
-            pin = pin[:pout.size]
-            frate = np.median(np.diff(pin))
-            if ind > 0:
-                """
-                assert that the pulses have the same length and that we don't miss frames during
-                the trial, the refresh rate of bpod is 100us
-                """
-                test1 = np.all(np.abs(1 - (pin - pout) / np.median(pin - pout)) < 0.1)
-                test2 = np.all(np.abs(np.diff(pin) - frate) <= 0.00011)
-                if not all([test1, test2]):
-                    n_out_of_sync += 1
-            # grow a list of cam times for ech trial
-            cam_times.append(pin)
-            n_frames += pin.size
-
-        if n_out_of_sync > 0:
-            _logger.warning(f"{n_out_of_sync} trials with bpod camera frame times not within"
-                            f" 10% of the expected sampling rate")
-
-        t_first_frame = np.array([c[0] for c in cam_times])
-        t_last_frame = np.array([c[-1] for c in cam_times])
-        frate = 1 / np.nanmedian(np.array([np.median(np.diff(c)) for c in cam_times]))
-        intertrial_duration = t_first_frame[1:] - t_last_frame[:-1]
-        intertrial_missed_frames = np.int32(np.round(intertrial_duration * frate)) - 1
-
-        # initialize the full times array
-        frame_times = np.zeros(n_frames + int(np.sum(intertrial_missed_frames)))
-        ii = 0
-        for trial, cam_time in enumerate(cam_times):
-            if cam_time is not None:
-                # populate first the recovered times within the trials
-                frame_times[ii: ii + cam_time.size] = cam_time
-                ii += cam_time.size
-            if trial == (len(cam_times) - 1):
-                break
-            # then extrapolate in-between
-            nmiss = intertrial_missed_frames[trial]
-            frame_times[ii: ii + nmiss] = (cam_time[-1] + intertrial_duration[trial] /
-                                           (nmiss + 1) * (np.arange(nmiss) + 1))
-            ii += nmiss
-        # import matplotlib.pyplot as plt
-        # plt.plot(np.diff(frame_times))
-        """
-        if we find a video file, get the number of frames and extrapolate the times
-         using the median frame rate as the video stops after the bpod
-        """
-        video_file = list(self.session_path.joinpath(
-            'raw_video_data').glob('_iblrig_leftCamera*.mp4'))
-        if video_file:
-            cap = cv2.VideoCapture(str(video_file[0]))
-            nframes = cap.get(cv2.CAP_PROP_FRAME_COUNT)
-            if nframes > len(frame_times):
-                to_app = (np.arange(int(nframes - frame_times.size),) + 1
-                          ) / frate + frame_times[-1]
-                frame_times = np.r_[frame_times, to_app]
-        assert(np.all(np.diff(frame_times) > 0))  # negative diffs implies a big problem
-        return frame_times
-
-
-def extract_all(session_path, save=False, bpod_trials=False, settings=False):
+def extract_all(session_path, save=False, bpod_trials=None, settings=None):
     if not bpod_trials:
         bpod_trials = raw.load_data(session_path)
     if not settings:
@@ -751,13 +656,13 @@ def extract_all(session_path, save=False, bpod_trials=False, settings=False):
         settings = {'IBLRIG_VERSION_TAG': '100.0.0'}
 
     base = [FeedbackType, ContrastLR, ProbabilityLeft, Choice, RepNum, RewardVolume,
-            FeedbackTimes, StimOnTimes, Intervals, ResponseTimes, GoCueTriggerTimes,
-            GoCueTimes]
+            FeedbackTimes, Intervals, ResponseTimes, GoCueTriggerTimes, GoCueTimes]
     # Version check
     if version.ge(settings['IBLRIG_VERSION_TAG'], '5.0.0'):
-        base.extend([StimOnTriggerTimes, CameraTimestamps])
+        base.extend([StimOnTriggerTimes, StimOnOffFreezeTimes, ItiInTimes,
+                     StimOffTriggerTimes, StimFreezeTriggerTimes, ErrorCueTriggerTimes])
     else:
-        base.extend([IncludedTrials, ItiDuration])
+        base.extend([IncludedTrials, ItiDuration, StimOnTimes])
 
     out, fil = run_extractor_classes(
         base, save=save, session_path=session_path, bpod_trials=bpod_trials, settings=settings)
