@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding:utf-8 -*-
-# @Author: Niccolò Bonacchi
+# @Author: Niccolò Bonacchi, Miles Wells
 # @Date: Monday, July 16th 2018, 1:28:46 pm
 """
 Raw Data Loader functions for PyBpod rig
@@ -10,15 +10,18 @@ Module contains one loader function per raw datafile
 import json
 import logging
 import wave
+from collections import OrderedDict
 from datetime import datetime
 from pathlib import Path
+from typing import Union
 
 import numpy as np
 import pandas as pd
 
-from alf.io import get_session_path
 from ibllib.io import jsonable
+from ibllib.io.video import assert_valid_label
 from ibllib.misc import version
+from ibllib.time import uncycle_pgts, convert_pgts
 
 _logger = logging.getLogger('ibllib')
 
@@ -74,7 +77,7 @@ def load_bpod(session_path):
     return load_settings(session_path), load_data(session_path)
 
 
-def load_data(session_path, time='absolute'):
+def load_data(session_path: Union[str, Path], time='absolute'):
     """
     Load PyBpod data files (.jsonable).
 
@@ -82,7 +85,7 @@ def load_data(session_path, time='absolute'):
     PyBpod timestamps are is seconds (s)
 
     :param session_path: Absolute path of session folder
-    :type session_path: str
+    :type session_path: str, Path
     :return: A list of len ntrials each trial being a dictionary
     :rtype: list of dicts
     """
@@ -100,14 +103,137 @@ def load_data(session_path, time='absolute'):
     return data
 
 
-def load_settings(session_path):
+def load_camera_ssv_times(session_path, camera: str):
+    """
+    Load the bonsai frame and camera timestamps from Camera.timestamps.ssv
+
+    NB: For some sessions the frame times are in the first column, in others the order is reversed.
+
+    :param session_path: Absolute path of session folder
+    :param camera: Name of the camera to load, e.g. 'left'
+    :return: array of datetimes, array of frame times in seconds
+    """
+    camera = assert_valid_label(camera)
+    file = Path(session_path) / 'raw_video_data' / f'_iblrig_{camera.lower()}Camera.timestamps.ssv'
+    assert file.exists()
+    # NB: Numpy has deprecated support for non-naive timestamps.
+    # Converting them is extremely slow: 6000 timestamps takes 0.8615s vs 0.0352s.
+    # from datetime import timezone
+    # c = {0: lambda x: datetime.fromisoformat(x).astimezone(timezone.utc).replace(tzinfo=None)}
+
+    # Determine the order of the columns by reading one line and testing whether the first value
+    # is an integer or not.
+    with open(file, 'r') as f:
+        line = f.readline()
+    type_map = OrderedDict(bonsai='<M8[ns]', camera='<u4')
+    try:
+        int(line.split(' ')[1])
+    except ValueError:
+        type_map.move_to_end('bonsai')
+    ssv_params = dict(names=type_map.keys(), dtype=','.join(type_map.values()), delimiter=' ')
+    ssv_times = np.genfromtxt(file, **ssv_params)  # np.loadtxt is slower for some reason
+    bonsai_times = ssv_times['bonsai']
+    camera_times = uncycle_pgts(convert_pgts(ssv_times['camera']))
+    return bonsai_times, camera_times
+
+
+def load_embedded_frame_data(session_path, label: str, raw=False):
+    """
+    Load the embedded frame count and GPIO for a given session.  If the file doesn't exist,
+    or is empty, None values are returned.
+    :param session_path: Absolute path of session folder
+    :param label: The specific video to load, one of ('left', 'right', 'body')
+    :param raw: If True the raw data are returned without preprocessing, otherwise frame count is
+    returned starting from 0 and the GPIO is returned as a dict of indices
+    :return: The frame count, GPIO
+    """
+    count = load_camera_frame_count(session_path, label, raw=raw)
+    gpio = load_camera_gpio(session_path, label, as_dicts=not raw)
+    return count, gpio
+
+
+def load_camera_frame_count(session_path, label: str, raw=True):
+    """
+    Load the embedded frame count for a given session.  If the file doesn't exist, or is empty,
+    a None value is returned.
+    :param session_path: Absolute path of session folder
+    :param label: The specific video to load, one of ('left', 'right', 'body')
+    :param raw: If True the raw data are returned without preprocessing, otherwise frame count is
+    returned starting from 0
+    :return: The frame count
+    """
+    if session_path is None:
+        return
+    raw_path = Path(session_path).joinpath('raw_video_data')
+
+    # Load frame count
+    count_file = raw_path / f'_iblrig_{assert_valid_label(label)}Camera.frame_counter.bin'
+    count = np.fromfile(count_file, dtype=np.float64).astype(int) if count_file.exists() else []
+    if len(count) == 0:
+        return
+    if not raw:
+        count -= count[0]  # start from zero
+    return count
+
+
+def load_camera_gpio(session_path, label: str, as_dicts=False):
+    """
+    Load the GPIO for a given session.  If the file doesn't exist, or is empty, a None value is
+    returned.
+
+    The raw binary file contains uint32 values (saved as doubles) where the first 4 bits
+    represent the state of each of the 4 GPIO pins. The array is expanded to an n x 4 array by
+    shifting each bit to the end and checking whether it is 0 (low state) or 1 (high state).
+
+    :param session_path: Absolute path of session folder
+    :param label: The specific video to load, one of ('left', 'right', 'body')
+    :param as_dicts: If False the raw data are returned boolean array with shape (n_frames, n_pins)
+     otherwise GPIO is returned as a list of dictionaries with keys ('indices', 'polarities').
+    :return: An nx4 boolean array where columns represent state of GPIO pins 1-4.
+     If as_dicts is True, a list of dicts is returned with keys ('indices', 'polarities'),
+     or None if the dictionary is empty.
+    """
+    if session_path is None:
+        return
+    raw_path = Path(session_path).joinpath('raw_video_data')
+
+    # Load pin state
+    GPIO_file = raw_path / f'_iblrig_{assert_valid_label(label)}Camera.GPIO.bin'
+    # This deals with missing and empty files the same
+    gpio = np.fromfile(GPIO_file, dtype=np.float64).astype(np.uint32) if GPIO_file.exists() else []
+    if len(gpio) == 0:
+        return [None] * 4 if as_dicts else None
+
+    # Check values make sense (4 pins = 16 possible values)
+    assert np.isin(gpio, np.left_shift(np.arange(2 ** 4, dtype=np.uint32), 32 - 4)).all()
+    # 4 pins represented as uint32. For each pin, shift its bit to the end and check the bit is set
+    gpio = (np.right_shift(np.tile(gpio, (4, 1)).T, np.arange(31, 27, -1)) & 0x1) == 1
+
+    if as_dicts:
+        if not gpio.any():
+            _logger.error('No GPIO changes')
+            return [None] * 4
+        # Find state changes for each pin and construct a dict of indices and polarities for each
+        edges = np.vstack((gpio[0, :], np.diff(gpio.astype(int), axis=0)))
+        # gpio = [(ind := np.where(edges[:, i])[0], edges[ind, i]) for i in range(4)]
+        # gpio = [dict(zip(('indices', 'polarities'), x)) for x in gpio_]  # py3.8
+        gpio = [{'indices': np.where(edges[:, i])[0],
+                 'polarities': edges[edges[:, i] != 0, i]}
+                for i in range(4)]
+        # Replace empty dicts with None
+        gpio = [None if x['indices'].size == 0 else x for x in gpio]
+
+    return gpio
+
+
+def load_settings(session_path: Union[str, Path]):
     """
     Load PyBpod Settings files (.json).
 
     [description]
 
     :param session_path: Absolute path of session folder
-    :type session_path: str
+    :type session_path: str, Path
     :return: Settings dictionary
     :rtype: dict
     """
@@ -386,7 +512,7 @@ def _clean_wheel_dataframe(data, label, path):
         _logger.warning(label + ' has missing/incomplete records \n %s', path)
     # first step is to re-interpret as numeric objects if not already done
     for col in data.columns:
-        if data[col].dtype == np.object and col not in ['bns_ts']:
+        if data[col].dtype == object and col not in ['bns_ts']:
             data[col] = pd.to_numeric(data[col], errors='coerce')
     # then drop Nans and duplicates
     data.dropna(inplace=True)
@@ -505,7 +631,7 @@ def sync_trials_robust(t0, t1, diff_threshold=0.001, drift_threshold_ppm=200, ma
             cdt = t0[i0 + 1] - t1[i1 + ii1]
         i0 += 1
     it0 = np.where(~np.isnan(ind))[0]
-    it1 = ind[it0].astype(np.int)
+    it1 = ind[it0].astype(int)
     ind0 = np.unique(np.r_[it0, it0 + 1])
     ind1 = np.unique(np.r_[it1, it1 + 1])
     if return_index:
@@ -514,70 +640,14 @@ def sync_trials_robust(t0, t1, diff_threshold=0.001, drift_threshold_ppm=200, ma
         return t0[ind0], t1[ind1]
 
 
-def get_task_extractor_type(task_name):
-    """
-    Splits the task name according to naming convention:
-    -   ignores everything
-    _iblrig_tasks_biasedChoiceWorld3.7.0 returns "biased"
-    _iblrig_tasks_trainingChoiceWorld3.6.0 returns "training'
-    :param task_name:
-    :return: one of ['biased', 'habituation', 'training', 'ephys', 'mock_ephys', 'sync_ephys']
-    """
-    if isinstance(task_name, Path):
-        try:
-            settings = load_settings(get_session_path(task_name))
-        except json.decoder.JSONDecodeError:
-            return
-        if settings:
-            task_name = settings.get('PYBPOD_PROTOCOL', None)
-        else:
-            return
-    if '_biasedChoiceWorld' in task_name:
-        return 'biased'
-    elif 'biasedScanningChoiceWorld' in task_name:
-        return 'biased'
-    elif 'biasedVisOffChoiceWorld' in task_name:
-        return 'biased'
-    elif '_habituationChoiceWorld' in task_name:
-        return 'habituation'
-    elif '_trainingChoiceWorld' in task_name:
-        return 'training'
-    elif 'ephysChoiceWorld' in task_name:
-        return 'ephys'
-    elif 'ephysMockChoiceWorld' in task_name:
-        return 'mock_ephys'
-    elif task_name and task_name.startswith('_iblrig_tasks_ephys_certification'):
-        return 'sync_ephys'
-
-
-def get_session_extractor_type(session_path):
-    """
-    From a session path, loads the settings file, finds the task and checks if extractors exist
-    task names examples:
-    :param session_path:
-    :return: bool
-    """
-    settings = load_settings(session_path)
-    if settings is None:
-        _logger.error(f'ABORT: No data found in "raw_behavior_data" folder {session_path}')
-        return False
-    extractor_type = get_task_extractor_type(settings['PYBPOD_PROTOCOL'])
-    if extractor_type:
-        return extractor_type
-    else:
-        _logger.warning(str(session_path) +
-                        f" No extractors were found for {extractor_type} ChoiceWorld")
-        return False
-
-
-def load_bpod_fronts(session_path: str, data: dict = False) -> list:
+def load_bpod_fronts(session_path: str, data: list = False) -> list:
     """load_bpod_fronts
     Loads BNC1 and BNC2 bpod channels times and polarities from session_path
 
     :param session_path: a valid session_path
     :type session_path: str
     :param data: pre-loaded raw data dict, defaults to False
-    :type data: dict, optional
+    :type data: list, optional
     :return: List of dicts BNC1 and BNC2 {"times": np.array, "polarities":np.array}
     :rtype: list
     """
