@@ -39,6 +39,13 @@ class Reader:
         if self.is_mtscomp:
             self._raw = mtscomp.Reader()
             self._raw.open(self.file_bin, self.file_bin.with_suffix('.ch'))
+            if self._raw.shape != (self.ns, self.nc):
+                ftsec = self._raw.shape[0] / self.fs
+                _logger.warning(f"{sglx_file} : meta data and compressed chunks dont checkout\n"
+                                f"File duration: expected {self.meta['fileTimeSecs']},"
+                                f" actual {ftsec}\n"
+                                f"Will attempt to fudge the meta-data information.")
+                self.meta['fileTimeSecs'] = ftsec
         else:
             if self.nc * self.ns * 2 != self.nbytes:
                 ftsec = self.file_bin.stat().st_size / 2 / self.nc / self.fs
@@ -151,15 +158,19 @@ class Reader:
         else:
             return self.read(nsel=_slice, csel=csel, sync=False)
 
-    def read_sync(self, _slice=slice(0, 10000), threshold=1.2):
+    def read_sync(self, _slice=slice(0, 10000), threshold=1.2, floor_percentile=10):
         """
         Reads all sync trace. Convert analog to digital with selected threshold and append to array
         :param _slice: samples slice
         :param threshold: (V) threshold for front detection, defaults to 1.2 V
+        :param floor_percentile: 10% removes the percentile value of the analog trace before
+         thresholding. This is to avoid DC offset drift
         :return: int8 array
         """
         digital = self.read_sync_digital(_slice)
         analog = self.read_sync_analog(_slice)
+        if analog is not None and floor_percentile:
+            analog -= np.percentile(analog, 10, axis=0)
         if analog is None:
             return digital
         analog[np.where(analog < threshold)] = 0
@@ -427,7 +438,7 @@ def split_sync(sync_tr):
 
 
 def get_neuropixel_version_from_folder(session_path):
-    ephys_files = glob_ephys_files(session_path)
+    ephys_files = glob_ephys_files(session_path, ext='meta')
     return get_neuropixel_version_from_files(ephys_files)
 
 
@@ -438,7 +449,7 @@ def get_neuropixel_version_from_files(ephys_files):
         return '3A'
 
 
-def glob_ephys_files(session_path, suffix='.meta', recursive=True, bin_exists=True):
+def glob_ephys_files(session_path, suffix='.meta', ext='bin', recursive=True, bin_exists=True):
     """
     From an arbitrary folder (usually session folder) gets the ap and lf files and labels
     Associated to the subfolders where they are
@@ -461,6 +472,7 @@ def glob_ephys_files(session_path, suffix='.meta', recursive=True, bin_exists=Tr
 
     :param bin_exists:
     :param suffix:
+    :param ext: file extension to look for, default 'bin' but could also be 'meta' or 'ch'
     :param recursive:
     :param session_path: folder, string or pathlib.Path
     :param glob_pattern: pattern to look recursively for (defaults to '*.ap.*bin)
@@ -475,24 +487,27 @@ def glob_ephys_files(session_path, suffix='.meta', recursive=True, bin_exists=Tr
     recurse = '**/' if recursive else ''
     ephys_files = []
     for raw_ephys_file in Path(session_path).glob(f'{recurse}*.ap{suffix}'):
-        raw_ephys_apfile = next(raw_ephys_file.parent.glob(raw_ephys_file.stem + '.*bin'), None)
+        raw_ephys_apfile = next(raw_ephys_file.parent.glob(raw_ephys_file.stem + f'.*{ext}'), None)
         if not raw_ephys_apfile and bin_exists:
             continue
-        elif not bin_exists:
+        elif not raw_ephys_apfile and ext != 'bin':
+            continue
+        elif not bin_exists and ext == 'bin':
             raw_ephys_apfile = raw_ephys_file.with_suffix('.bin')
         # first get the ap file
         ephys_files.extend([Bunch({'label': None, 'ap': None, 'lf': None, 'path': None})])
         ephys_files[-1].ap = raw_ephys_apfile
         # then get the corresponding lf file if it exists
         lf_file = raw_ephys_apfile.parent / raw_ephys_apfile.name.replace('.ap.', '.lf.')
-        ephys_files[-1].lf = next(lf_file.parent.glob(lf_file.stem + '.*bin'), None)
+        ephys_files[-1].lf = next(lf_file.parent.glob(lf_file.stem + f'.*{ext}'), None)
         # finally, the label is the current directory except if it is bare in raw_ephys_data
         ephys_files[-1].label = get_label(raw_ephys_apfile)
         ephys_files[-1].path = raw_ephys_apfile.parent
     # for 3b probes, need also to get the nidq dataset type
     for raw_ephys_file in Path(session_path).rglob(f'{recurse}*.nidq{suffix}'):
-        raw_ephys_nidqfile = next(raw_ephys_file.parent.glob(raw_ephys_file.stem + '.*bin'), None)
-        if not bin_exists:
+        raw_ephys_nidqfile = next(raw_ephys_file.parent.glob(raw_ephys_file.stem + f'.*{ext}'),
+                                  None)
+        if not bin_exists and ext == 'bin':
             raw_ephys_nidqfile = raw_ephys_file.with_suffix('.bin')
         ephys_files.extend([Bunch({'label': get_label(raw_ephys_file),
                                    'nidq': raw_ephys_nidqfile,
