@@ -1,13 +1,14 @@
 import logging
+from datetime import datetime
 from pathlib import Path
+import pkg_resources
 import re
 import subprocess
-import pkg_resources
-from datetime import datetime
+import sys
+import traceback
 
+from ibllib.io.extractors.base import get_session_extractor_type, get_pipeline
 from ibllib.pipes import ephys_preprocessing, training_preprocessing, tasks
-import ibllib.io.raw_data_loaders as rawio
-import ibllib.exceptions
 from ibllib.time import date2isostr
 
 import oneibl.registration as registration
@@ -24,30 +25,35 @@ def _get_lab(one):
         return [la['name'] for la in lab]
 
 
+def _run_command(cmd):
+    process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
+                               stderr=subprocess.PIPE)
+    info, error = process.communicate()
+    if process.returncode != 0:
+        return None
+    else:
+        return info.decode('utf-8').strip()
+
+
+def _get_volume_usage(vol, label=''):
+    cmd = f'df {vol}'
+    res = _run_command(cmd)
+    # size_list = ['/dev/sdc1', '1921802500', '1427128132', '494657984', '75%', '/datadisk']
+    size_list = re.split(' +', res.split('\n')[-1])
+    fac = 1024 ** 2
+    d = {'total': int(size_list[1]) / fac,
+         'used': int(size_list[2]) / fac,
+         'available': int(size_list[3]) / fac,
+         'volume': size_list[5]}
+    return {f"{label}_{k}": d[k] for k in d}
+
+
 def report_health(one):
     """
     Get a few indicators and label the json field of the corresponding lab with them
     """
-    def _run_command(cmd):
-        process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
-                                   stderr=subprocess.PIPE)
-        info, error = process.communicate()
-        if process.returncode != 0:
-            return None
-        else:
-            return info.decode('utf-8').strip()
-
-    def _get_volume_usage(vol, label=''):
-        cmd = f'df {vol}'
-        res = _run_command(cmd)
-        # size_list = ['/dev/sdc1', '1921802500', '1427128132', '494657984', '75%', '/datadisk']
-        size_list = re.split(' +', res.split('\n')[-1])
-        fac = 1024 ** 2
-        d = {'total': int(size_list[1]) / fac, 'used': int(size_list[2]) / fac,
-             'volume': size_list[5]}
-        return {f"{label}_{k}": d[k] for k in d}
-
-    status = {'ibllib_version': pkg_resources.get_distribution("ibllib").version,
+    status = {'python_version': sys.version,
+              'ibllib_version': pkg_resources.get_distribution("ibllib").version,
               'phylib_version': pkg_resources.get_distribution("phylib").version,
               'local_time': date2isostr(datetime.now())}
     status.update(_get_volume_usage('/mnt/s0/Data', 'raid'))
@@ -74,38 +80,42 @@ def job_creator(root_path, one=None, dry=False, rerun=False, max_md5_size=None):
     if not one:
         one = ONE()
     rc = registration.RegistrationClient(one=one)
-    flag_files = list(Path(root_path).glob('**/extract_me.flag'))
-    flag_files += list(Path(root_path).glob('**/raw_session.flag'))
+    flag_files = list(Path(root_path).glob('**/raw_session.flag'))
     all_datasets = []
     for flag_file in flag_files:
         session_path = flag_file.parent
         _logger.info(f'creating session for {session_path}')
         if dry:
             continue
-        flag_file.unlink()
-        # if the subject doesn't exist in the database, skip
+
         try:
+            # if the subject doesn't exist in the database, skip
             rc.create_session(session_path)
-        except ibllib.exceptions.AlyxSubjectNotFound:
+            files, dsets = registration.register_session_raw_data(
+                session_path, one=one, max_md5_size=max_md5_size)
+            if dsets is not None:
+                all_datasets.extend(dsets)
+            pipeline = get_pipeline(session_path)
+            if pipeline == 'training':
+                pipe = training_preprocessing.TrainingExtractionPipeline(session_path, one=one)
+            # only start extracting ephys on a raw_session.flag
+            elif pipeline == 'ephys' and flag_file.name == 'raw_session.flag':
+                pipe = ephys_preprocessing.EphysExtractionPipeline(session_path, one=one)
+            else:
+                _logger.info(f"Session type {get_session_extractor_type(session_path)}"
+                             f"as no matching pipeline pattern {session_path}")
+                continue
+            if rerun:
+                rerun__status__in = '__all__'
+            else:
+                rerun__status__in = ['Waiting']
+            pipe.create_alyx_tasks(rerun__status__in=rerun__status__in)
+            flag_file.unlink()
+        except BaseException:
+            _logger.error(traceback.format_exc())
+            _logger.warning(f"Creating session / registering raw datasets {session_path} errored")
             continue
-        files, dsets = registration.register_session_raw_data(
-            session_path, one=one, max_md5_size=max_md5_size)
-        if dsets is not None:
-            all_datasets.extend(dsets)
-        session_type = rawio.get_session_extractor_type(session_path)
-        if session_type in ['biased', 'habituation', 'training']:
-            pipe = training_preprocessing.TrainingExtractionPipeline(session_path, one=one)
-        # only start extracting ephys on a raw_session.flag
-        elif session_type in ['ephys'] and flag_file.name == 'raw_session.flag':
-            pipe = ephys_preprocessing.EphysExtractionPipeline(session_path, one=one)
-        else:
-            _logger.info(f"Session type {session_type} as no matching extractor {session_path}")
-            return
-        if rerun:
-            rerun__status__in = '__all__'
-        else:
-            rerun__status__in = ['Waiting']
-        pipe.create_alyx_tasks(rerun__status__in=rerun__status__in)
+
     return all_datasets
 
 
