@@ -1,8 +1,15 @@
 import abc
 import concurrent.futures
+import json
 import logging
 import os
+import fnmatch
+import re
+from functools import wraps
 from pathlib import Path, PurePath
+import shutil
+from typing import Any, Sequence, Union, Optional, List, Dict
+from uuid import UUID
 
 import requests
 import tqdm
@@ -11,22 +18,29 @@ import numpy as np
 
 import oneibl.params
 import oneibl.webclient as wc
-from alf.io import (AlfBunch, get_session_path, is_uuid_string,
-                    load_file_content, remove_uuid_file)
-from ibllib.io import hashfile
+import alf.io as alfio
+from alf.files import is_valid, alf_parts
+# from ibllib.misc.exp_ref import is_exp_ref
+from ibllib.exceptions import \
+    ALFMultipleObjectsFound, ALFObjectNotFound, ALFMultipleCollectionsFound
+from ibllib.io import hashfile, spikeglx
 from ibllib.misc import pprint
 from oneibl.dataclass import SessionDataInfo
 from brainbox.io import parquet
-from brainbox.core import ismember, ismember2d
+from brainbox.numerical import ismember, ismember2d, find_first_2d
 
 _logger = logging.getLogger('ibllib')
 
+
+def Listable(t): return Union[t, Sequence[t]]  # noqa
+
+
 NTHREADS = 4  # number of download threads
 
 _ENDPOINTS = {  # keynames are possible input arguments and values are actual endpoints
     'data': 'dataset-types',
-    'dataset': 'dataset-types',
-    'datasets': 'dataset-types',
+    'dataset': 'datasets',
+    'datasets': 'datasets',
     'dataset-types': 'dataset-types',
     'dataset_types': 'dataset-types',
     'dataset-type': 'dataset-types',
@@ -53,21 +67,6 @@ _SESSION_FIELDS = {  # keynames are possible input arguments and values are actu
     'end_time': 'end_time',
     'end-time': 'end_time'}
 
-_LIST_KEYWORDS = dict(_SESSION_FIELDS, **{
-    'all': 'all',
-    'data': 'dataset-type',
-    'dataset': 'dataset-type',
-    'datasets': 'dataset-type',
-    'dataset-types': 'dataset-type',
-    'dataset_types': 'dataset-type',
-    'dataset-type': 'dataset-type',
-    'dataset_type': 'dataset-type',
-    'dtypes': 'dataset-type',
-    'dtype': 'dataset-type',
-    'labs': 'lab',
-    'lab': 'lab'
-})
-
 SEARCH_TERMS = {  # keynames are possible input arguments and values are actual fields
     'data': 'dataset_types',
     'dataset': 'dataset_types',
@@ -116,11 +115,27 @@ def _ses2pandas(ses, dtypes=None):
     return col
 
 
+def parse_id(method):
+    """
+    Ensures the input experiment identifier is an experiment UUID string
+    :param method: An ONE method whose second arg is an experiment id
+    :return: A wrapper function that parses the id to the expected string
+    """
+    @wraps(method)
+    def wrapper(self, id, *args, **kwargs):
+        id = self.to_eid(id)
+        return method(self, id, *args, **kwargs)
+    return wrapper
+
+
 class OneAbstract(abc.ABC):
 
     def __init__(self, username=None, password=None, base_url=None, cache_dir=None, silent=None):
         # get parameters override if inputs provided
         self._par = oneibl.params.get(silent=silent)
+        # can delete those 2 lines from mid January 2021
+        if self._par.HTTP_DATA_SERVER == 'http://ibl.flatironinstitute.org':
+            self._par = self._par.set("HTTP_DATA_SERVER", "https://ibl.flatironinstitute.org")
         self._par = self._par.set('ALYX_LOGIN', username or self._par.ALYX_LOGIN)
         self._par = self._par.set('ALYX_URL', base_url or self._par.ALYX_URL)
         self._par = self._par.set('ALYX_PWD', password or self._par.ALYX_PWD)
@@ -139,7 +154,7 @@ class OneAbstract(abc.ABC):
         From a Session ID and dataset types, queries Alyx database, downloads the data
         from Globus, and loads into numpy array. Single session only
         """
-        if is_uuid_string(eid):
+        if alfio.is_uuid_string(eid):
             eid = '/sessions/' + eid
         eid_str = eid[-36:]
         # if no dataset_type is provided:
@@ -155,7 +170,7 @@ class OneAbstract(abc.ABC):
         # load the files content in variables if requested
         if not download_only:
             for ind, fil in enumerate(dc.local_path):
-                dc.data[ind] = load_file_content(fil)
+                dc.data[ind] = alfio.load_file_content(fil)
         # parse output arguments
         if dclass_output:
             return dc
@@ -180,6 +195,7 @@ class OneAbstract(abc.ABC):
         # if empty in parameter file, do not allow and set default
         if not cache_dir:
             cache_dir = str(PurePath(Path.home(), "Downloads", "FlatIron"))
+        assert cache_dir
         return cache_dir
 
     def _make_dataclass_offline(self, eid, dataset_types=None, cache_dir=None, **kwargs):
@@ -193,6 +209,63 @@ class OneAbstract(abc.ABC):
         df = df[ismember(df['dataset_type'], dataset_types)[0]]
         return SessionDataInfo.from_pandas(df, self._get_cache_dir(cache_dir))
 
+    def path_from_eid(self, eid: str) -> Optional[Listable(Path)]:
+        """
+        From an experiment id or a list of experiment ids, gets the local cache path
+        :param eid: eid (UUID) or list of UUIDs
+        :return: eid or list of eids
+        """
+        # If eid is a list of eIDs recurse through list and return the results
+        if isinstance(eid, list):
+            path_list = []
+            for p in eid:
+                path_list.append(self.path_from_eid(p))
+            return path_list
+        # If not valid return None
+        if not alfio.is_uuid_string(eid):
+            print(eid, " is not a valid eID/UUID string")
+            return
+        if self._cache.size == 0:
+            return
+
+        # load path from cache
+        ic = find_first_2d(
+            self._cache[['eid_0', 'eid_1']].to_numpy(), parquet.str2np(eid))
+        if ic is not None:
+            ses = self._cache.iloc[ic]
+            return Path(self._par.CACHE_DIR).joinpath(
+                ses['lab'], 'Subjects', ses['subject'], ses['start_time'].isoformat()[:10],
+                str(ses['number']).zfill(3))
+
+    def eid_from_path(self, path_obj):
+        """
+        From a local path, gets the experiment id
+        :param path_obj: local path or list of local paths
+        :return: eid or list of eids
+        """
+        # If path_obj is a list recurse through it and return a list
+        if isinstance(path_obj, list):
+            path_obj = [Path(x) for x in path_obj]
+            eid_list = []
+            for p in path_obj:
+                eid_list.append(self.eid_from_path(p))
+            return eid_list
+        # else ensure the path ends with mouse,date, number
+        path_obj = Path(path_obj)
+        session_path = alfio.get_session_path(path_obj)
+        # if path does not have a date and a number, or cache is empty return None
+        if session_path is None or self._cache.size == 0:
+            return None
+
+        # fetch eid from cache
+        ind = ((self._cache['subject'] == session_path.parts[-3]) &
+               (self._cache['start_time'].apply(
+                   lambda x: x.isoformat()[:10] == session_path.parts[-2])) &
+               (self._cache['number']) == int(session_path.parts[-1]))
+        ind = np.where(ind.to_numpy())[0]
+        if ind.size > 0:
+            return parquet.np2str(self._cache[['eid_0', 'eid_1']].iloc[ind[0]])
+
     @abc.abstractmethod
     def _make_dataclass(self, eid, dataset_types=None, cache_dir=None, **kwargs):
         pass
@@ -266,71 +339,30 @@ class OneAlyx(OneAbstract):
         out = self.alyx.rest('dataset-types', 'read', dataset_type)
         print(out['description'])
 
-    def list(self, eid=None, keyword='dataset-type', details=False):
+    def list(self, eid: Optional[Union[str, Path, UUID]] = None, details=False
+             ) -> Union[List, Dict[str, str]]:
         """
-        From a Session ID, queries Alyx database for datasets-types related to a session.
+        From a Session ID, queries Alyx database for datasets related to a session.
 
-        :param eid: Experiment ID, for IBL this is the UUID String of the Session as per Alyx
-         database. Example: '698361f6-b7d0-447d-a25d-42afdef7a0da'
-         If None, returns the set of possible values. Only for the following keys:
-         ('users', 'dataset-types', subjects')
-        :type eid: str or list of strings
+        :param eid: Experiment session uuid str
+        :type eid: str
 
-        :param keyword: The attribute to be listed.
-        :type keyword: str
+        :param details: If false returns a list of path, otherwise returns the REST dictionary
+        :type eid: bool
 
-        :param details: returns a second argument with a full dictionary to provide context
-        :type details: bool
-
-        :return: list of strings, plus list of dictionaries if details option selected
-        :rtype:  list, list
-
-        for a list of keywords, use the methods `one.search_terms()`
+        :return: list of strings or dict of lists if details is True
+        :rtype:  list, dict
         """
-        # check and validate the input keyword
-        if keyword.lower() not in set(_LIST_KEYWORDS.keys()):
-            raise KeyError("The field: " + keyword + " doesn't exist in the Session model." +
-                           "\n Here is a list of expected values: " +
-                           str(set(_LIST_KEYWORDS.values())))
-        keyword = _LIST_KEYWORDS[keyword.lower()]  # accounts for possible typos
+        if not eid:
+            return [x['name'] for x in self.alyx.rest('dataset-types', 'list')]
 
-        # recursive call for cases where a list is provided
-        if isinstance(eid, list):
-            out = []
-            for e in eid:
-                out.append(self.list(e, keyword=keyword, details=details))
-            if details and (keyword != 'dataset-type'):
-                return [[o[0] for o in out], [o[1] for o in out]]
-            else:
-                return out
-
-        #  this is for basic endpoints queries: dataset-type, users, and subjects with None
-        if eid is None:
-            out = self._ls(table=keyword)
-            if details:
-                return out
-            else:
-                return out[0]
-
-        # this is a query about datasets: need to unnest session info through the load function
-        if keyword == 'dataset-type':
-            dses = self.load(eid, dataset_types='__all__', dry_run=True)
-            dlist = list(sorted(set(dses.dataset_type)))
-            if details:
-                return dses
-            else:
-                return dlist
-
-        # get the session information
-        ses = self.alyx.rest('sessions', 'read', eid)
-
-        if keyword.lower() == 'all':
-            return [ses]
-        elif details:
-            return ses[keyword], ses
-        else:
-            return ses[keyword]
+        # Session specific list
+        dsets = self.alyx.rest('datasets', 'list', session=eid, exists=True)
+        if not details:
+            dsets = sorted([Path(dset['collection']).joinpath(dset['name']) for dset in dsets])
+        return dsets
 
+    @parse_id
     def load(self, eid, dataset_types=None, dclass_output=False, dry_run=False, cache_dir=None,
              download_only=False, clobber=False, offline=False, keep_uuid=False):
         """
@@ -365,47 +397,107 @@ class OneAlyx(OneAbstract):
                                     dry_run=dry_run, cache_dir=cache_dir, keep_uuid=keep_uuid,
                                     download_only=download_only, clobber=clobber, offline=offline)
 
-    def load_dataset(self, eid, dataset_type, **kwargs):
+    @parse_id
+    def load_dataset(self,
+                     eid: Union[str, Path, UUID],
+                     dataset: str,
+                     collection: Optional[str] = None,
+                     download_only: bool = False) -> Any:
         """
         Load a single dataset from a Session ID and a dataset type.
 
-        :param eid: Experiment ID, for IBL this is the UUID of the Session as per Alyx
-         database. Could be a full Alyx URL:
-         'http://localhost:8000/sessions/698361f6-b7d0-447d-a25d-42afdef7a0da' or only the UUID:
-         '698361f6-b7d0-447d-a25d-42afdef7a0da'.
-        :type eid: str
-        :param dataset_type: Alyx dataset type to be returned.
-        :type dataset_types: str
+        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
+        details dict or Path
+        :param dataset: The ALF dataset to load.  Supports asterisks as wildcards.
+        :param collection:  The collection to which the object belongs, e.g. 'alf/probe01'.
+        For IBL this is the relative path of the file from the session root.
+        Supports asterisks as wildcards.
+        :param download_only: When true the data are downloaded and the file path is returned
+        :return: dataset or a Path object if download_only is true
 
-        :return: A numpy array.
-        :rtype: numpy array
+        Examples:
+            intervals = one.load_dataset(eid, '_ibl_trials.intervals.npy')
+            intervals = one.load_dataset(eid, '*trials.intervals*')
+            filepath = one.load_dataset(eid '_ibl_trials.intervals.npy', download_only=True)
+            spikes = one.load_dataset(eid 'spikes.times.npy', collection='alf/probe01')
         """
-        return self._load(eid, dataset_types=[dataset_type], **kwargs)[0]
+        search_str = 'name__regex,' + dataset.replace('.', r'\.').replace('*', '.*')
+        if collection:
+            search_str += ',collection__regex,' + collection.replace('*', '.*')
+        results = self.alyx.rest('datasets', 'list', session=eid, django=search_str, exists=True)
 
-    def load_object(self, eid, obj, **kwargs):
+        # Get filenames of returned ALF files
+        collection_set = {x['collection'] for x in results}
+        if len(collection_set) > 1:
+            raise ALFMultipleCollectionsFound('Matching dataset belongs to multiple collections:' +
+                                              ', '.join(collection_set))
+        if len(results) > 1:
+            raise ALFMultipleObjectsFound('The following matching datasets were found: ' +
+                                          ', '.join(x['name'] for x in results))
+        if len(results) == 0:
+            raise ALFObjectNotFound(f'Dataset "{dataset}" not found on Alyx')
+
+        filename = self.download_dataset(results[0])
+        assert filename is not None, 'failed to download dataset'
+
+        return filename if download_only else alfio.load_file_content(filename)
+
+    @parse_id
+    def load_object(self,
+                    eid: Union[str, Path, UUID],
+                    obj: str,
+                    collection: Optional[str] = 'alf',
+                    download_only: bool = False,
+                    **kwargs) -> Union[alfio.AlfBunch, List[Path]]:
         """
         Load all attributes of an ALF object from a Session ID and an object name.
 
-        :param eid: Experiment ID, for IBL this is the UUID of the Session as per Alyx
-         database. Could be a full Alyx URL:
-         'http://localhost:8000/sessions/698361f6-b7d0-447d-a25d-42afdef7a0da' or only the UUID:
-         '698361f6-b7d0-447d-a25d-42afdef7a0da'.
-        :type eid: str
-        :param obj: Alyx object to load.
-        :type obj: str
+        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
+        details dict or Path
+        :param obj: The ALF object to load.  Supports asterisks as wildcards.
+        :param collection:  The collection to which the object belongs, e.g. 'alf/probe01'.
+        Supports asterisks as wildcards.
+        :param download_only: When true the data are downloaded and the file paths are returned
+        :param kwargs: Optional filters for the ALF objects, including namespace and timescale
+        :return: An ALF bunch or if download_only is True, a list of Paths objects
 
-        :return: A dictionary-like structure with one key per dataset type for the requested
-         object, and a NumPy array per value.
-        :rtype: AlfBunch instance
+        Examples:
+        load_object(eid, '*moves')
+        load_object(eid, 'trials')
+        load_object(eid, 'spikes', collection='*probe01')
         """
-        dataset_types = [dst for dst in self.list(eid) if dst.startswith(obj)]
-        if len(dataset_types) == 0:
-            _logger.warning(f"{eid} does not contain any {obj} object datasets")
-            return
-        dsets = self._load(eid, dataset_types=dataset_types, **kwargs)
-        return AlfBunch({
-            '.'.join(dataset_types[i].split('.')[1:]): dsets[i]
-            for i in range(len(dataset_types))})
+        # Filter server-side by collection and dataset name
+        search_str = 'name__regex,' + obj.replace('*', '.*')
+        if collection and collection != 'all':
+            search_str += ',collection__regex,' + collection.replace('*', '.*')
+        results = self.alyx.rest('datasets', 'list', exists=True, session=eid, django=search_str)
+        pattern = re.compile(fnmatch.translate(obj))
+
+        # Further refine by matching object part of ALF datasets
+        def match(r):
+            return is_valid(r['name']) and pattern.match(alf_parts(r['name'])[1])
+
+        # Get filenames of returned ALF files
+        returned_obj = {alf_parts(x['name'])[1] for x in results if match(x)}
+
+        # Validate result before loading
+        if len(returned_obj) > 1:
+            raise ALFMultipleObjectsFound('The following matching objects were found: ' +
+                                          ', '.join(returned_obj))
+        elif len(returned_obj) == 0:
+            raise ALFObjectNotFound(f'ALF object "{obj}" not found on Alyx')
+        collection_set = {x['collection'] for x in results if match(x)}
+        if len(collection_set) > 1:
+            raise ALFMultipleCollectionsFound('Matching object belongs to multiple collections:' +
+                                              ', '.join(collection_set))
+
+        # Download and optionally load the datasets
+        out_files = self.download_datasets(x for x in results if match(x))
+        assert not any(x is None for x in out_files), 'failed to download object'
+        if download_only:
+            return out_files
+        else:
+            return alfio.load_object(out_files[0].parent, obj, **kwargs)
 
     def _load_recursive(self, eid, **kwargs):
         """
@@ -428,6 +520,36 @@ class OneAlyx(OneAbstract):
                     out.append(self._load(e, **kwargs)[0])
             return out
 
+    def to_eid(self,
+               id: Listable(Union[str, Path, UUID, dict]) = None,
+               cache_dir: Optional[Union[str, Path]] = None) -> Listable(str):
+        if isinstance(id, (list, tuple)):  # Recurse
+            return [self.to_eid(i, cache_dir) for i in id]
+        if isinstance(id, UUID):
+            return str(id)
+        # elif is_exp_ref(id):
+        #     return ref2eid(id, one=self)
+        elif isinstance(id, dict):
+            assert {'subject', 'number', 'start_time', 'lab'}.issubset(id)
+            root = Path(self._get_cache_dir(cache_dir))
+            id = root.joinpath(
+                id['lab'],
+                'Subjects', id['subject'],
+                id['start_time'][:10],
+                ('%03d' % id['number']))
+
+        if alfio.is_session_path(id):
+            return self.eid_from_path(id)
+        elif isinstance(id, str):
+            if len(id) > 36:
+                id = id[-36:]
+            if not alfio.is_uuid_string(id):
+                raise ValueError('Invalid experiment ID')
+            else:
+                return id
+        else:
+            raise ValueError('Unrecognized experiment ID')
+
     def _make_dataclass(self, eid, dataset_types=None, cache_dir=None, dry_run=False,
                         clobber=False, offline=False, keep_uuid=False):
         # if the input as an UUID, add the beginning of URL to it
@@ -475,22 +597,20 @@ class OneAlyx(OneAbstract):
         assert (isinstance(table, str))
         table_field_names = {
             'dataset-types': 'name',
+            'datasets': 'name',
             'users': 'username',
             'subjects': 'nickname',
             'labs': 'name'}
         if not table or table not in list(set(_ENDPOINTS.keys())):
             raise KeyError("The attribute/endpoint: " + table + " doesn't exist \n" +
                            "possible values are " + str(set(_ENDPOINTS.values())))
-        full_out = []
-        list_out = []
-        for ind, tab in enumerate(_ENDPOINTS):
-            if tab == table:
-                field_name = table_field_names[_ENDPOINTS[tab]]
-                full_out.append(self.alyx.get('/' + _ENDPOINTS[tab]))
-                list_out.append([f[field_name] for f in full_out[-1]])
+
+        field_name = table_field_names[_ENDPOINTS[table]]
+        full_out = self.alyx.get('/' + _ENDPOINTS[table])
+        list_out = [f[field_name] for f in full_out]
         if verbose:
             pprint(list_out)
-        return list_out[0], full_out[0]
+        return list_out, full_out
 
     # def search(self, dataset_types=None, users=None, subjects=None, date_range=None,
     #            lab=None, number=None, task_protocol=None, details=False):
@@ -619,8 +739,11 @@ class OneAlyx(OneAbstract):
         if isinstance(dset, str):
             url = dset
         else:
-            url = next((fr['data_url'] for fr in dset['file_records'] if fr['data_url']), None)
+            url = next((fr['data_url'] for fr in dset['file_records']
+                        if fr['data_url'] and fr['exists']), None)
         if not url:
+            str_dset = Path(dset['collection']).joinpath(dset['name'])
+            _logger.warning(f"{str_dset} exist flag or url not found in Alyx")
             return
         assert url.startswith(self._par.HTTP_DATA_SERVER), \
             ('remote protocol and/or hostname does not match HTTP_DATA_SERVER parameter:\n' +
@@ -646,7 +769,6 @@ class OneAlyx(OneAbstract):
         """
         Downloads a single file from an HTTP webserver
         :param url:
-        :param cache_dir:
         :param clobber: (bool: False) overwrites local dataset if any
         :param offline:
         :param keep_uuid:
@@ -657,8 +779,8 @@ class OneAlyx(OneAbstract):
         Path(target_dir).mkdir(parents=True, exist_ok=True)
         local_path = str(target_dir) + os.sep + os.path.basename(url)
         if not keep_uuid:
-            local_path = remove_uuid_file(local_path, dry=True)
-        if Path(local_path).exists() and not offline:
+            local_path = alfio.remove_uuid_file(local_path, dry=True)
+        if Path(local_path).exists():
             # the local file hash doesn't match the dataset table cached hash
             hash_mismatch = hash and hashfile.md5(Path(local_path)) != hash
             file_size_mismatch = file_size and Path(local_path).stat().st_size != file_size
@@ -668,11 +790,11 @@ class OneAlyx(OneAbstract):
         # if there is no cached file, download
         else:
             clobber = True
-        if clobber:
+        if clobber and not offline:
             local_path, md5 = wc.http_download_file(
                 url, username=self._par.HTTP_DATA_SERVER_LOGIN,
                 password=self._par.HTTP_DATA_SERVER_PWD, cache_dir=str(target_dir),
-                clobber=clobber, offline=offline, return_md5=True)
+                clobber=clobber, return_md5=True)
             # post download, if there is a mismatch between Alyx and the newly downloaded file size
             # or hash flag the offending file record in Alyx for database maintenance
             hash_mismatch = hash and md5 != hash
@@ -682,7 +804,7 @@ class OneAlyx(OneAbstract):
         if keep_uuid:
             return local_path
         else:
-            return remove_uuid_file(local_path)
+            return alfio.remove_uuid_file(local_path)
 
     @staticmethod
     def search_terms():
@@ -711,7 +833,7 @@ class OneAlyx(OneAbstract):
         """
         oneibl.params.setup()
 
-    def path_from_eid(self, eid: str, use_cache=True) -> Path:
+    def path_from_eid(self, eid: str, use_cache: bool = True) -> Listable(Path):
         """
         From an experiment id or a list of experiment ids, gets the local cache path
         :param eid: eid (UUID) or list of UUIDs
@@ -725,19 +847,15 @@ class OneAlyx(OneAbstract):
                 path_list.append(self.path_from_eid(p))
             return path_list
         # If not valid return None
-        if not is_uuid_string(eid):
+        if not alfio.is_uuid_string(eid):
             print(eid, " is not a valid eID/UUID string")
             return
 
         # first try avoid hitting the database
         if self._cache.size > 0 and use_cache:
-            ic = parquet.find_first_2d(
-                self._cache[['eid_0', 'eid_1']].to_numpy(), parquet.str2np(eid))
-            if ic is not None:
-                ses = self._cache.iloc[ic]
-                return Path(self._par.CACHE_DIR).joinpath(
-                    ses['lab'], 'Subjects', ses['subject'], ses['start_time'].isoformat()[:10],
-                    str(ses['number']).zfill(3))
+            cache_path = super().path_from_eid(eid)
+            if cache_path:
+                return cache_path
 
         # if it wasn't successful, query Alyx
         ses = self.alyx.rest('sessions', 'list', django=f'pk,{eid}')
@@ -748,7 +866,7 @@ class OneAlyx(OneAbstract):
                 ses[0]['lab'], 'Subjects', ses[0]['subject'], ses[0]['start_time'][:10],
                 str(ses[0]['number']).zfill(3))
 
-    def eid_from_path(self, path_obj, use_cache=True):
+    def eid_from_path(self, path_obj: Union[str, Path], use_cache: bool = True) -> Listable(Path):
         """
         From a local path, gets the experiment id
         :param path_obj: local path or list of local paths
@@ -764,20 +882,15 @@ class OneAlyx(OneAbstract):
             return eid_list
         # else ensure the path ends with mouse,date, number
         path_obj = Path(path_obj)
-        session_path = get_session_path(path_obj)
+        session_path = alfio.get_session_path(path_obj)
         # if path does not have a date and a number return None
         if session_path is None:
             return None
 
         # try the cached info to possibly avoid hitting database
-        if self._cache.size > 0 and use_cache:
-            ind = ((self._cache['subject'] == session_path.parts[-3]) &
-                   (self._cache['start_time'].apply(
-                       lambda x: x.isoformat()[:10] == session_path.parts[-2])) &
-                   (self._cache['number']) == int(session_path.parts[-1]))
-            ind = np.where(ind.to_numpy())[0]
-            if ind.size > 0:
-                return parquet.np2str(self._cache[['eid_0', 'eid_1']].iloc[ind[0]])
+        cache_eid = super().eid_from_path(path_obj)
+        if cache_eid:
+            return cache_eid
 
         # if not search for subj, date, number XXX: hits the DB
         uuid = self.search(subjects=session_path.parts[-3],
@@ -787,7 +900,35 @@ class OneAlyx(OneAbstract):
         # Return the uuid if any
         return uuid[0] if uuid else None
 
-    def get_details(self, eid, full=False):
+    def url_from_path(self, filepath):
+        """
+        Given a local file path, returns the URL of the remote file.
+        :param filepath: A local file path
+        :return: A URL string
+        """
+        eid = self.eid_from_path(filepath)
+        try:
+            dataset, = self.alyx.rest('datasets', 'list', session=eid, name=Path(filepath).name)
+        except ValueError:
+            raise ALFObjectNotFound(f'File record for {filepath} not found on Alyx')
+        return next(
+            r['data_url'] for r in dataset['file_records'] if r['data_url'] and r['exists'])
+
+    @parse_id
+    def datasets_from_type(self, eid, dataset_type, full=False):
+        """
+        Get list of datasets belonging to a given dataset type for a given session
+        :param eid: Experiment session identifier; may be a UUID, URL, experiment reference string
+        details dict or Path
+        :param dataset_type: A dataset type, e.g. camera.times
+        :param full: If True, a dictionary of details is returned for each dataset
+        :return: A list of datasets belonging to that session's dataset type
+        """
+        restriction = f'session__id,{eid},dataset_type__name,{dataset_type}'
+        datasets = self.alyx.rest('datasets', 'list', django=restriction)
+        return datasets if full else [d['name'] for d in datasets]
+
+    def get_details(self, eid: str, full: bool = False):
         """ Returns details of eid like from one.search, optional return full
         session details.
         """
@@ -798,7 +939,7 @@ class OneAlyx(OneAbstract):
                 details_list.append(self.get_details(p, full=full))
             return details_list
         # If not valid return None
-        if not is_uuid_string(eid):
+        if not alfio.is_uuid_string(eid):
             print(eid, " is not a valid eID/UUID string")
             return
         # load all details
@@ -840,8 +981,7 @@ class OneAlyx(OneAbstract):
             eq = np.logical_and(heq, feq)
             # update new hash / filesizes
             if not np.all(eq):
-                self._cache.iloc[icache].loc[:, ['file_size', 'hash']] = \
-                    pqt_dsets.iloc[np.where(isin)[0]].loc[:, ['file_size', 'hash']]
+                self._cache.iloc[icache, 4:6] = pqt_dsets.iloc[np.where(isin)[0], 4:6].to_numpy()
                 save = True
             # append datasets that haven't been found
             if not np.all(isin):
@@ -855,6 +995,85 @@ class OneAlyx(OneAbstract):
             # if this gets too big, look into saving only when destroying the ONE object
             parquet.save(self._cache_file, self._cache)
 
+    def download_raw_partial(self, url_cbin, url_ch, first_chunk=0, last_chunk=0):
+        assert str(url_cbin).endswith('.cbin')
+        assert str(url_ch).endswith('.ch')
+
+        relpath = Path(url_cbin.replace(self._par.HTTP_DATA_SERVER, '.')).parents[0]
+        target_dir = Path(self._get_cache_dir(None), relpath)
+        Path(target_dir).mkdir(parents=True, exist_ok=True)
+
+        # First, download the .ch file if necessary
+        if isinstance(url_ch, Path):
+            ch_file = url_ch
+        else:
+            ch_file = Path(wc.http_download_file(
+                url_ch,
+                username=self._par.HTTP_DATA_SERVER_LOGIN,
+                password=self._par.HTTP_DATA_SERVER_PWD,
+                cache_dir=target_dir, clobber=True, return_md5=False))
+            ch_file = alfio.remove_uuid_file(ch_file)
+        ch_file_stream = ch_file.with_suffix('.stream.ch')
+        # Load the .ch file.
+        with open(ch_file, 'r') as f:
+            cmeta = json.load(f)
+
+        # Get the first byte and number of bytes to download.
+        i0 = cmeta['chunk_bounds'][first_chunk]
+        ns_stream = cmeta['chunk_bounds'][last_chunk + 1] - i0
+
+        # if the cached version happens to be the same as the one on disk, just load it
+        if ch_file_stream.exists():
+            with open(ch_file_stream, 'r') as f:
+                cmeta_stream = json.load(f)
+            if (cmeta_stream.get('chopped_first_sample', None) == i0 and
+                    cmeta_stream.get('chopped_total_samples', None) == ns_stream):
+                return spikeglx.Reader(ch_file_stream.with_suffix('.cbin'))
+        else:
+            shutil.copy(ch_file, ch_file_stream)
+        assert ch_file_stream.exists()
+
+        # prepare the metadata file
+        cmeta['chunk_bounds'] = cmeta['chunk_bounds'][first_chunk:last_chunk + 2]
+        cmeta['chunk_bounds'] = [_ - i0 for _ in cmeta['chunk_bounds']]
+        assert len(cmeta['chunk_bounds']) >= 2
+        assert cmeta['chunk_bounds'][0] == 0
+
+        first_byte = cmeta['chunk_offsets'][first_chunk]
+        cmeta['chunk_offsets'] = cmeta['chunk_offsets'][first_chunk:last_chunk + 2]
+        cmeta['chunk_offsets'] = [_ - first_byte for _ in cmeta['chunk_offsets']]
+        assert len(cmeta['chunk_offsets']) >= 2
+        assert cmeta['chunk_offsets'][0] == 0
+        n_bytes = cmeta['chunk_offsets'][-1]
+        assert n_bytes > 0
+
+        # Save the chopped chunk bounds and ossets.
+        cmeta['sha1_compressed'] = None
+        cmeta['sha1_uncompressed'] = None
+        cmeta['chopped'] = True
+        cmeta['chopped_first_sample'] = i0
+        cmeta['chopped_total_samples'] = ns_stream
+
+        with open(ch_file_stream, 'w') as f:
+            json.dump(cmeta, f, indent=2, sort_keys=True)
+
+        # Download the requested chunks
+        cbin_local_path = wc.http_download_file(
+            url_cbin,
+            username=self._par.HTTP_DATA_SERVER_LOGIN,
+            password=self._par.HTTP_DATA_SERVER_PWD,
+            cache_dir=target_dir, clobber=True, return_md5=False,
+            chunks=(first_byte, n_bytes))
+        cbin_local_path = alfio.remove_uuid_file(cbin_local_path)
+        cbin_local_path_renamed = cbin_local_path.with_suffix('.stream.cbin')
+        cbin_local_path.rename(cbin_local_path_renamed)
+        assert cbin_local_path_renamed.exists()
+
+        shutil.copy(cbin_local_path.with_suffix('.meta'),
+                    cbin_local_path_renamed.with_suffix('.meta'))
+        reader = spikeglx.Reader(cbin_local_path_renamed)
+        return reader
+
 
 def _validate_date_range(date_range):
     """
