@@ -2,6 +2,8 @@ from pathlib import Path
 import json
 import datetime
 import logging
+
+import ibllib.io.extractors.base
 from dateutil import parser as dateparser
 import re
 
@@ -24,6 +26,7 @@ REGISTRATION_GLOB_PATTERNS = ['alf/**/*.*',
                               'raw_ephys_data/**/_iblrig_*.*',
                               'raw_ephys_data/**/_spikeglx_*.*',
                               'raw_ephys_data/**/_iblqc_*.*',
+                              'spikesorters/**/_kilosort_*.*'
                               ]
 
 
@@ -38,7 +41,7 @@ def _check_filename_for_registration(full_file, patterns):
 
 
 def register_dataset(file_list, one=None, created_by=None, repository=None, server_only=False,
-                     versions=False, dry=False, max_md5_size=None):
+                     versions=None, revisions=None, default=True, dry=False, max_md5_size=None):
     """
     Registers a set of files belonging to a session only on the server
     :param file_list: (list of pathlib.Path or pathlib.Path)
@@ -47,6 +50,8 @@ def register_dataset(file_list, one=None, created_by=None, repository=None, serv
     :param repository: optional: (string) name of the repository in Alyx
     :param server_only: optional: (bool) if True only creates on the Flatiron (defaults to False)
     :param versions: optional (list of strings): versions tags (defaults to ibllib version)
+    :param revisions: optional (list of strings): revision name (defaults to no revision)
+    :param default: optional (bool) whether to set as default dataset (defaults to True)
     :param dry: (bool) False by default
     :param verbose: (bool) logs
     :param max_md5_size: (int) maximum file in bytes to compute md5 sum (always compute if Npne)
@@ -59,12 +64,21 @@ def register_dataset(file_list, one=None, created_by=None, repository=None, serv
         return
     elif not isinstance(file_list, list):
         file_list = [Path(file_list)]
+
     assert len(set([alf.io.get_session_path(f) for f in file_list])) == 1
     assert all([Path(f).exists() for f in file_list])
-    if not versions:
-        versions = [version.ibllib() for _ in file_list]
+    if versions is None:
+        versions = version.ibllib()
+    if isinstance(versions, str):
+        versions = [versions for _ in file_list]
+    assert isinstance(versions, list) and len(versions) == len(file_list)
+
+    if revisions is None:
+        revisions = [None for _ in file_list]
     else:
-        assert isinstance(versions, list) and len(versions) == len(file_list)
+        if isinstance(revisions, str):
+            revisions = [revisions for _ in file_list]
+    assert isinstance(revisions, list) and len(revisions) == len(file_list)
 
     # computing the md5 can be very long, so this is an option to skip if the file is bigger
     # than a certain threshold
@@ -83,7 +97,9 @@ def register_dataset(file_list, one=None, created_by=None, repository=None, serv
          'server_only': server_only,
          'hashes': hashes,
          'filesizes': [p.stat().st_size for p in file_list],
-         'versions': versions}
+         'versions': versions,
+         'revisions': revisions,
+         'default': default}
     if not dry:
         if one is None:
             one = ONE()
@@ -245,11 +261,12 @@ class RegistrationClient:
 
         # if nothing found create a new session in Alyx
         task_protocol = md['PYBPOD_PROTOCOL'] + md['IBLRIG_VERSION_TAG']
+        alyx_procedure = _alyx_procedure_from_task(task_protocol)
         if not session:
             ses_ = {'subject': subject['nickname'],
                     'users': [username],
                     'location': md['PYBPOD_BOARD'],
-                    'procedures': [_alyx_procedure_from_task(task_protocol)],
+                    'procedures': [] if alyx_procedure is None else [alyx_procedure],
                     'lab': subject['lab'],
                     # 'project': project['name'],
                     'type': 'Experiment',
@@ -314,13 +331,13 @@ class RegistrationClient:
                 raise e
             # extract the relative path of the file
             rel_path = Path(str(fn)[str(fn).find(str(gen_rel_path)):])
-            F.append(str(rel_path.relative_to(gen_rel_path)))
+            F.append(str(rel_path.relative_to(gen_rel_path).as_posix()))
             file_sizes.append(fn.stat().st_size)
             md5s.append(hashfile.md5(fn) if fn.stat().st_size < 1024 ** 3 else None)
             _logger.info('Registering ' + str(fn))
 
         r_ = {'created_by': username,
-              'path': str(gen_rel_path),
+              'path': str(gen_rel_path.as_posix()),
               'filenames': F,
               'hashes': md5s,
               'filesizes': file_sizes,
@@ -330,16 +347,22 @@ class RegistrationClient:
 
 
 def _alyx_procedure_from_task(task_protocol):
-    task_str = raw.get_task_extractor_type(task_protocol)
+    task_type = ibllib.io.extractors.base.get_task_extractor_type(task_protocol)
+    return _alyx_procedure_from_task_type(task_type)
+
+
+def _alyx_procedure_from_task_type(task_type):
     lookup = {'biased': 'Behavior training/tasks',
+              'biased_opto': 'Behavior training/tasks',
               'habituation': 'Behavior training/tasks',
               'training': 'Behavior training/tasks',
               'ephys': 'Ephys recording with acute probe(s)',
+              'ephys_biased_opto': 'Ephys recording with acute probe(s)',
+              'ephys_training': 'Ephys recording with acute probe(s)',
               'mock_ephys': 'Ephys recording with acute probe(s)',
               'sync_ephys': 'Ephys recording with acute probe(s)'}
-    if task_str not in lookup:
-        return
-    return lookup[task_str]
+    if task_type in lookup:
+        return lookup[task_type]
 
 
 def _register_bool(fn, file_list):
