@@ -5,13 +5,14 @@ import os
 import re
 import urllib.request
 from collections.abc import Mapping
-from pathlib import Path
+from pathlib import Path, PurePosixPath
 import hashlib
 
 import requests
 
 from ibllib.misc import pprint, print_progress
 
+SDSC_ROOT_PATH = PurePosixPath('/mnt/ibl')
 _logger = logging.getLogger('ibllib')
 
 
@@ -21,6 +22,7 @@ class _PaginatedResponse(Mapping):
     Provides cache functionality
     PaginatedResponse(alyx, response)
     """
+
     def __init__(self, alyx, rep):
         self.alyx = alyx
         self.count = rep['count']
@@ -52,6 +54,88 @@ class _PaginatedResponse(Mapping):
             yield self.__getitem__(i)
 
 
+def sdsc_globus_path_from_dataset(dset):
+    """
+    :param dset: dset dictionary or list of dictionaries from ALyx rest endpoint
+    Returns SDSC globus file path from a dset record or a list of dsets records from REST
+    """
+    return _path_from_dataset(dset, root_path=PurePosixPath('/'), repository=None, uuid=True)
+
+
+def globus_path_from_dataset(dset, repository=None, uuid=False):
+    """
+    Returns local one file path from a dset record or a list of dsets records from REST
+    :param dset: dset dictionary or list of dictionaries from ALyx rest endpoint
+    :param repository: (optional) repository name of the file record (if None, will take
+     the first filerecord with an URL)
+    """
+    return _path_from_dataset(dset, root_path=PurePosixPath('/'), repository=repository, uuid=uuid)
+
+
+def one_path_from_dataset(dset, one_cache):
+    """
+    Returns local one file path from a dset record or a list of dsets records from REST
+    :param dset: dset dictionary or list of dictionaries from ALyx rest endpoint
+    :param one_cache: the one cache directory
+    """
+    return _path_from_dataset(dset, root_path=one_cache, uuid=False)
+
+
+def sdsc_path_from_dataset(dset, root_path=SDSC_ROOT_PATH):
+    """
+    Returns sdsc file path from a dset record or a list of dsets records from REST
+    :param dset: dset dictionary or list of dictionaries from ALyx rest endpoint
+    :param root_path: (optional) the prefix path such as one download directory or sdsc root
+    """
+    return _path_from_dataset(dset, root_path=root_path, uuid=True)
+
+
+def _path_from_dataset(dset, root_path=None, repository=None, uuid=False):
+    """
+    returns the local file path from a dset record from a REST query
+    :param dset: dset dictionary or list of dictionaries from ALyx rest endpoint
+    :param root_path: (optional) the prefix path such as one download directory or sdsc root
+    :param repository:
+    :param uuid: (optional bool) if True, will add UUID before the file extension
+    :return: Path or list of Path
+    """
+    if isinstance(dset, list):
+        return [_path_from_dataset(d) for d in dset]
+    if repository:
+        fr = next((fr for fr in dset['file_records'] if fr['data_repository'] == repository))
+    else:
+        fr = next((fr for fr in dset['file_records'] if fr['data_url']))
+    uuid = dset['url'][-36:] if uuid else None
+    return _path_from_filerecord(fr, root_path=root_path, uuid=uuid)
+
+
+def _path_from_filerecord(fr, root_path=SDSC_ROOT_PATH, uuid=None):
+    """
+    Returns a data file Path constructed from an Alyx file record.  The Path type returned
+    depends on the type of root_path: If root_path is a string a Path object is returned,
+    otherwise if the root_path is a PurePath, the same path type is returned.
+    :param fr: An Alyx file record dict
+    :param root_path: An optional root path
+    :param uuid: An optional UUID to add to the file name
+    :return: A filepath as a pathlib object
+    """
+    import alf.io
+    if isinstance(fr, list):
+        return [_path_from_filerecord(f) for f in fr]
+    repo_path = fr['data_repository_path']
+    repo_path = repo_path[repo_path.startswith('/'):]  # remove starting / if any
+    # repo_path = (p := fr['data_repository_path'])[p[0] == '/':]  # py3.8 Remove slash at start
+    file_path = PurePosixPath(repo_path, fr['relative_path'])
+    if root_path:
+        # NB: By checking for string we won't cast any PurePaths
+        if isinstance(root_path, str):
+            root_path = Path(root_path)
+        file_path = root_path / file_path
+    if uuid:
+        file_path = alf.io.add_uuid_string(file_path, uuid)
+    return file_path
+
+
 def http_download_file_list(links_to_file_list, **kwargs):
     """
     Downloads a list of files from the flat Iron from a list of links.
@@ -68,8 +152,8 @@ def http_download_file_list(links_to_file_list, **kwargs):
     return file_names_list
 
 
-def http_download_file(full_link_to_file, *, clobber=False, offline=False,
-                       username='', password='', cache_dir='', return_md5=False):
+def http_download_file(full_link_to_file, chunks=None, *, clobber=False,
+                       username='', password='', cache_dir='', return_md5=False, headers=None):
     """
     :param full_link_to_file: http link to the file.
     :type full_link_to_file: str
@@ -81,6 +165,7 @@ def http_download_file(full_link_to_file, *, clobber=False, offline=False,
     :type password: str
     :param cache_dir: [''] directory in which files are cached; defaults to user's
      Download directory.
+    :param: headers: [{}] additional headers to add to the request (auth tokens etc..)
     :type cache_dir: str
 
     :return: (str) a list of the local full path of the downloaded files.
@@ -99,8 +184,6 @@ def http_download_file(full_link_to_file, *, clobber=False, offline=False,
     # do not overwrite an existing file unless specified
     if not clobber and os.path.exists(file_name):
         return (file_name, hashfile.md5(file_name)) if return_md5 else file_name
-    elif offline:
-        return (file_name, hashfile.md5(file_name)) if return_md5 else file_name
 
     # This should be the base url you wanted to access.
     baseurl = os.path.split(str(full_link_to_file))[0]
@@ -117,9 +200,20 @@ def http_download_file(full_link_to_file, *, clobber=False, offline=False,
     opener = urllib.request.build_opener(auth)
     urllib.request.install_opener(opener)
 
+    # Support for partial download.
+    req = urllib.request.Request(full_link_to_file)
+    if chunks is not None:
+        first_byte, n_bytes = chunks
+        req.add_header("Range", "bytes=%d-%d" % (first_byte, first_byte + n_bytes - 1))
+
+    # add additional headers
+    if headers is not None:
+        for k in headers:
+            req.add_header(k, headers[k])
+
     # Open the url and get the length
     try:
-        u = urllib.request.urlopen(full_link_to_file)
+        u = urllib.request.urlopen(req)
     except urllib.error.HTTPError as e:
         _logger.error(f"{str(e)} {full_link_to_file}")
         raise e
@@ -206,9 +300,6 @@ class AlyxClient(metaclass=UniqueSingletons):
     Class that implements simple GET/POST wrappers for the Alyx REST API
     http://alyx.readthedocs.io/en/latest/api.html
     """
-    _token = ''  # These class params are not being used!!
-    _headers = ''
-    _rest_schemes = ''
 
     def __init__(self, **kwargs):
         """
@@ -230,16 +321,18 @@ class AlyxClient(metaclass=UniqueSingletons):
         self._headers['Accept'] = 'application/json'
         self._obj_id = id(self)
 
-    def _generic_request(self, reqfunction, rest_query, data=None):
-        # if the data is a dictionary, it has to be converted to json text
-        if isinstance(data, dict) or isinstance(data, list):
-            data = json.dumps(data)
+    def _generic_request(self, reqfunction, rest_query, data=None, files=None):
         # makes sure the base url is the one from the instance
         rest_query = rest_query.replace(self._base_url, '')
         if not rest_query.startswith('/'):
             rest_query = '/' + rest_query
-        _logger.debug(self._base_url + rest_query)
-        r = reqfunction(self._base_url + rest_query, stream=True, headers=self._headers, data=data)
+        _logger.debug(f"{self._base_url + rest_query}, headers: {self._headers}")
+        headers = self._headers.copy()
+        if files is None:
+            data = json.dumps(data) if isinstance(data, dict) or isinstance(data, list) else data
+            headers['Content-Type'] = 'application/json'
+        r = reqfunction(self._base_url + rest_query, stream=True, headers=headers,
+                        data=data, files=files)
         if r and r.status_code in (200, 201):
             return json.loads(r.text)
         elif r and r.status_code == 204:
@@ -271,8 +364,7 @@ class AlyxClient(metaclass=UniqueSingletons):
             raise Exception('Alyx authentication error. Check your credentials')
         self._headers = {
             'Authorization': 'Token {}'.format(list(self._token.values())[0]),
-            'Accept': 'application/json',
-            'Content-Type': 'application/json'}
+            'Accept': 'application/json'}
 
     def delete(self, rest_query):
         """
@@ -288,6 +380,15 @@ class AlyxClient(metaclass=UniqueSingletons):
         """
         return self._generic_request(requests.delete, rest_query)
 
+    def download_file(self, url, **kwargs):
+        """
+        Downloads a file on the Alyx server from a filerecord REST field URL
+        :param url: full url of the file
+        :param kwargs: webclient.http_download_file parameters
+        :return: local path of downloaded file
+        """
+        return http_download_file(url, headers=self._headers, **kwargs)
+
     def get(self, rest_query):
         """
         Sends a GET request to the Alyx server. Will raise an exception on any status_code
@@ -309,7 +410,7 @@ class AlyxClient(metaclass=UniqueSingletons):
                 rep = rep['results']
         return rep
 
-    def patch(self, rest_query, data=None):
+    def patch(self, rest_query, data=None, files=None):
         """
         Sends a PATCH request to the Alyx server.
         For the dictionary contents, refer to:
@@ -317,16 +418,15 @@ class AlyxClient(metaclass=UniqueSingletons):
 
         :param rest_query: (required)the endpoint as full or relative URL
         :type rest_query: str
-        :param data: json encoded string or dictionary
+        :param data: json encoded string or dictionary (cf.requests)
         :type data: None, dict or str
+        :param files: dictionary / tuple (cf.requests)
 
         :return: response object
         """
-        if isinstance(data, dict):
-            data = json.dumps(data)
-        return self._generic_request(requests.patch, rest_query, data=data)
+        return self._generic_request(requests.patch, rest_query, data=data, files=files)
 
-    def post(self, rest_query, data=None):
+    def post(self, rest_query, data=None, files=None):
         """
         Sends a POST request to the Alyx server.
         For the dictionary contents, refer to:
@@ -336,12 +436,13 @@ class AlyxClient(metaclass=UniqueSingletons):
         :type rest_query: str
         :param data: dictionary or json encoded string
         :type data: None, dict or str
+        :param files: dictionary / tuple (cf.requests)
 
         :return: response object
         """
-        return self._generic_request(requests.post, rest_query, data=data)
+        return self._generic_request(requests.post, rest_query, data=data, files=files)
 
-    def put(self, rest_query, data=None):
+    def put(self, rest_query, data=None, files=None):
         """
         Sends a PUT request to the Alyx server.
         For the dictionary contents, refer to:
@@ -351,12 +452,13 @@ class AlyxClient(metaclass=UniqueSingletons):
         :type rest_query: str
         :param data: dictionary or json encoded string
         :type data: None, dict or str
+        :param files: dictionary / tuple (cf.requests)
 
         :return: response object
         """
-        return self._generic_request(requests.put, rest_query, data=data)
+        return self._generic_request(requests.put, rest_query, data=data, files=files)
 
-    def rest(self, url=None, action=None, id=None, data=None, **kwargs):
+    def rest(self, url=None, action=None, id=None, data=None, files=None, **kwargs):
         """
         alyx_client.rest(): lists endpoints
         alyx_client.rest(endpoint): lists actions for endpoint
@@ -371,12 +473,15 @@ class AlyxClient(metaclass=UniqueSingletons):
             alyx.client.rest('subjects', 'update', id='nickname', data=sub_dict)
             alyx.client.rest('subjects', 'partial_update', id='nickname', data=sub_dict)
             alyx.client.rest('subjects', 'delete', id='nickname')
+            alyx.client.rest('notes', 'create', data=nd, files={'image': open(image_file, 'rb')})
 
         :param url: endpoint name
         :param action: 'list', 'create', 'read', 'update', 'partial_update', 'delete'
         :param id: lookup string for actions 'read', 'update', 'partial_update', and 'delete'
         :param data: data dictionary for actions 'update', 'partial_update' and 'create'
-        :param ``**kwargs``: filter as per the REST documentation
+        :param files: if file upload
+        :param ``**kwargs``: filter as per the Alyx REST documentation
+            cf. https://alyx.internationalbrainlab.org/docs/
         :return: list of queried dicts ('list') or dict (other actions)
         """
         # if endpoint is None, list available endpoints
@@ -424,9 +529,14 @@ class AlyxClient(metaclass=UniqueSingletons):
             assert(endpoint_scheme[action]['action'] == 'get')
             # add to url data if it is a string
             if id:
-                url = url + id
+                # this is a special case of the list where we query an uuid. Usually read is better
+                if 'django' in kwargs.keys():
+                    kwargs['django'] = kwargs['django'] + ','
+                else:
+                    kwargs['django'] = ""
+                kwargs['django'] = f"{kwargs['django']}pk,{id}"
             # otherwise, look for a dictionary of filter terms
-            elif kwargs:
+            if kwargs:
                 url += '?'
                 for k in kwargs.keys():
                     if isinstance(kwargs[k], str):
@@ -442,16 +552,16 @@ class AlyxClient(metaclass=UniqueSingletons):
             return self.get('/' + endpoint + '/' + id.split('/')[-1])
         elif action == 'create':
             assert(endpoint_scheme[action]['action'] == 'post')
-            return self.post('/' + endpoint, data=data)
+            return self.post('/' + endpoint, data=data, files=files)
         elif action == 'delete':
             assert(endpoint_scheme[action]['action'] == 'delete')
             return self.delete('/' + endpoint + '/' + id.split('/')[-1])
         elif action == 'partial_update':
             assert(endpoint_scheme[action]['action'] == 'patch')
-            return self.patch('/' + endpoint + '/' + id.split('/')[-1], data=data)
+            return self.patch('/' + endpoint + '/' + id.split('/')[-1], data=data, files=files)
         elif action == 'update':
             assert(endpoint_scheme[action]['action'] == 'put')
-            return self.put('/' + endpoint + '/' + id.split('/')[-1], data=data)
+            return self.put('/' + endpoint + '/' + id.split('/')[-1], data=data, files=files)
 
     # JSON field interface convenience methods
     def _check_inputs(self, endpoint: str) -> None:
@@ -505,7 +615,7 @@ class AlyxClient(metaclass=UniqueSingletons):
         values (uses the dict.update() method)
 
         Example:
-        one.alyx.json_field_update("sessions", "eid_str", "extended_qc" {"key": value})
+        one.alyx.json_field_update("sessions", "eid_str", "extended_qc", {"key": value})
 
         :param endpoint: endpoint to hit
         :type endpoint: str
